//===================================================================================================================================
// Deckadance Remote Control Casio XW-J1 Implementation
// Created by Daire O'Neill, Vestax Corporation
//Version 2.0 
//27th August 2013
//===================================================================================================================================
#include "dd_commands.h"

//Globals
int ScratchA_On = 0;
int SamplerA_On = 0,SamplerB_On = 0; //for setting LEDs to sampler / cue state

void OnMidiMessageReceived(const string &in name, int status, int data, int value)
{
    // stack
    float value01 = value / 127.0;
	float isDown = (value > 0) ? 1.0 : 0.0;
	
	string val; //for debug
	
	/****************************************************************************************************/
	// NN's Deck A
	/****************************************************************************************************/
	
	if(status == 0x90) { //No bitmasks, too messy
		
		switch(data){
		
		//Play
		case 0x2E: if(value > 0) set(PLAY_PAUSE_A); break;
		//Main cue
		case 0x2F: if(value > 0)set(CUE_A); break;		
		//Cue stutter (CUP)
		case 0x30: set(CUE_PLAY_A,isDown); break;	
		//Sync
		case 0x31: if(value > 0) set(SYNC_LOCK_A); break;		
		//Key lock
		case 0x2D: if(value > 0) set(KEY_LOCK_A);break;
		//Jog scratch on
		case 0x3B: set(JOG_SCRATCH_ON_A,isDown);break;		
		//Set hot cues
		case 0x26: if(value > 0) set(HOT_CUE_1_A); break;	
		case 0x27: if(value > 0) set(HOT_CUE_2_A); break;
		case 0x28: if(value > 0) set(HOT_CUE_3_A); break;
		//Delete hot cues
		case 0x2A: if(value > 0) set(DELETE_HOT_CUE_1_A);break;				
		case 0x2B: if(value > 0) set(DELETE_HOT_CUE_2_A);break;			
		case 0x2C: if(value > 0) set(DELETE_HOT_CUE_3_A);break;			
		//Sampler
		case 0x62: if(value > 0) set(SAMPLER_PLAY_PAD_1,1); break;
		case 0x63: if(value > 0) set(SAMPLER_PLAY_PAD_2,1); break;
		case 0x64: if(value > 0) set(SAMPLER_PLAY_PAD_3,1); break;
		case 0x65: if(value > 0) set(SAMPLER_PLAY_PAD_4,1); break;
		case 0x66: if(value > 0) set(SAMPLER_PLAY_PAD_5,1); break;
		case 0x67: if(value > 0) set(SAMPLER_PLAY_PAD_6,1); break;
		//Loop	
		case 0x23: if(value > 0) set(LOOP_HALF_A); break; //half
		case 0x24: if(value > 0) set(LOOP_A); break;// loop on
		case 0x25: if(value > 0) set(LOOP_DOUBLE_A); break; //double
		//FX on
		case 0x22: if(value > 0) set(EFFECT_ON_1_GROUP_A); break;
		//Pitch bend -
		case 0x20: set(PITCH_BEND_NUDGE_MINUS_A,value01);break;
		//Pitch bend +
		case 0x21: set(PITCH_BEND_NUDGE_PLUS_A,value01); break;
		case 0x32: if(value > 0) set(PFL_A);break;
		//Sampler On flag
		case 0x33: //both messages used
		case 0x68: if(value > 0) (SamplerA_On == 0) ? (SamplerA_On = 1) : (SamplerA_On = 0); break;	
			
	 }//switch data
	}//if((status & 0xF0) == 0x90) 
	
	
	/****************************************************************************************************/
	// CC's Deck A
	/****************************************************************************************************/
	if(status == 0xB0) { //Deck A
		switch(data)		{
		
		// Volume
		case 0x1C: set(VOLUME_A, value01); break;		
		//EQ, Gain and Filter
		case 0x1A: set(FILTER_A, value01); break;
		case 0x14: set(EQ_LOW_A, value01); break;
		case 0x13: set(EQ_MID_A, value01); break;            
		case 0x12: set(EQ_HIGH_A, value01); break;
		case 0x11: set(GAIN_A, value01); break;
		//Tempo fader, touch strip, Fx knob
		case 0x1B: set(PITCH_A,(1.0-value01)); break;
		case 0x1D: set(SEEK_A,value01);break;
		case 0x10: set(EFFECT_PARAM_1_GROUP_A,value01);break;
	    }//switch(data)
	}// if((status & 0xF0) == 0xB0)
	
	if(status == 0xB2)
	{ 
		switch(data)
		{	//Jog scratch
			case 0x10: set(JOG_A, value - 64.0, -0.01, 0.1); break;
		}
	}	
	/****************************************************************************************************/
	// NN's Deck B
	/****************************************************************************************************/
	if(status == 0x91) { 
		
		switch(data){
		
		//Play
		case 0x2E: if(value > 0) set(PLAY_PAUSE_B); break;
		//Main cue
		case 0x2F: if(value > 0) set(CUE_B); break;		
		//Cue stutter (CUP)
		case 0x30: set(CUE_PLAY_B,isDown); break;		
		//Sync
		case 0x31: if(value > 0) set(SYNC_LOCK_B); break;
		//Key lock
		case 0x2D: if(value > 0) set(KEY_LOCK_B);break;
		//Jog scratch on
		case 0x3B: set(JOG_SCRATCH_ON_B,isDown);break;		
		//Set hot cues
		case 0x26: if(value > 0) set(HOT_CUE_1_B); break;	
		case 0x27: if(value > 0) set(HOT_CUE_2_B); break;
		case 0x28: if(value > 0) set(HOT_CUE_3_B); break;
		//Delete hot cues
		case 0x2A: if(value > 0) set(DELETE_HOT_CUE_1_B);break;			
		case 0x2B: if(value > 0) set(DELETE_HOT_CUE_2_B);break;			
		case 0x2C: if(value > 0) set(DELETE_HOT_CUE_3_B);break;				
		//Sampler
		case 0x62: if(value > 0) set(SAMPLER_PLAY_PAD_7,1); break;	
		case 0x63: if(value > 0) set(SAMPLER_PLAY_PAD_8,1); break;
		case 0x64: if(value > 0) set(SAMPLER_PLAY_PAD_9,1); break;
		case 0x65: if(value > 0) set(SAMPLER_PLAY_PAD_10,1); break;
		case 0x66: if(value > 0) set(SAMPLER_PLAY_PAD_11,1); break;
		case 0x67: if(value > 0) set(SAMPLER_PLAY_PAD_12,1); break;
		//Loop	
		case 0x23: if(value > 0) set(LOOP_HALF_B); break; //half
		case 0x24: if(value > 0) set(LOOP_B); break;// loop on
		case 0x25: if(value > 0) set(LOOP_DOUBLE_B); break; //double	
		//FX on
		case 0x22: if(value > 0) set(EFFECT_ON_1_GROUP_B); break;
		//Pitch bend -
		case 0x20: 	set(PITCH_BEND_NUDGE_MINUS_B,value01);break;
		//Pitch bend +
		case 0x21: set(PITCH_BEND_NUDGE_PLUS_B,value01); break;
		case 0x32: if(value > 0) set(PFL_B);break;
		//Sampler On flag
		case 0x33: //both messages used
		case 0x68: if(value > 0) (SamplerB_On == 0) ? (SamplerB_On = 1) : (SamplerB_On = 0);break;	
			
	}//switch data
	}//if((status & 0xF0) == 0x91) 
	
	/****************************************************************************************************/
	// CC's Deck B
	/****************************************************************************************************/
	if(status == 0xB1) { //Deck B
		switch(data) {
		
		// Volume
		case 0x16: set(VOLUME_B, value01); break; 	
		//EQ, Gain and Filter
		case 0x1B: set(FILTER_B, value01); break;
		case 0x15: set(EQ_LOW_B, value01); break;
		case 0x12: set(EQ_MID_B, value01); break;            
		case 0x13: set(EQ_HIGH_B, value01); break;
		case 0x1C: set(GAIN_B, value01); break;
		//Tempo fader, touch strip, Fx knob
		case 0x11: set(PITCH_B,(1.0-value01)); break; //invert
		case 0x17: set(SEEK_B,value01);break;
		case 0x14: set(EFFECT_PARAM_1_GROUP_B,value01);break;
	    }//switch(data)
	}// if((status & 0xF0) == 0xB0)
	
	if(status == 0xB2)
	{ 
		switch(data)
		{	//Jog scratch
			case 0x11: set(JOG_B, value - 64.0, -0.01, 0.1); break;
		}
	}
	
	/****************************************************************************************************/
	// Misc controls
	/****************************************************************************************************/
	if(status == 0xB4)
	{ 
		switch(data)
		{
			//Crossfader
			case 0x10: set(CROSSFADER, value01); break;	
		}
	}//if(status == 0xB4)
	
	if(status  == 0x92)
	{
		switch(data)
		{	
			//Middle section
			case 0x20: set(PLAYLIST_PREVIEW);set(PREVIEW_PLAY_PAUSE);break;
			case 0x21: if(value > 0) set(PLAYLIST_UP); break;
			case 0x22: if(value > 0) set(PLAYLIST_DOWN); break; 
			case 0x23: if(value > 0) set(PLAYLIST_LOAD_A); break;
			case 0x24: if(value > 0) set(PLAYLIST_LOAD_B); break; 
			case 0x25: if(value > 0) set(DISK_RECORDER); break;
		}
	}
	
	}//OnMidiMessageReceived


void OnTimerCallback()
{	
	//Deck A LEDs
	(get(PLAY_PAUSE_A) > 0) ? sendMidiMessage(0x90, 0x2E, 0x7F) : sendMidiMessage(0x90, 0x2E, 0x00);
	//Not in spec document, but to set green LED for sample pad use 0x56-0x5C
	if(SamplerA_On == 1)((get(SAMPLER_PLAY_PAD_1)>0) ? sendMidiMessage(0x90, 0x56, 0x7F) : sendMidiMessage(0x90, 0x56, 0x00)); 
	if(SamplerA_On == 1)((get(SAMPLER_PLAY_PAD_2)>0) ? sendMidiMessage(0x90, 0x57, 0x7F) : sendMidiMessage(0x90, 0x57, 0x00));
	if(SamplerA_On == 1)((get(SAMPLER_PLAY_PAD_3)>0) ? sendMidiMessage(0x90, 0x58, 0x7F) : sendMidiMessage(0x90, 0x58, 0x00));
	if(SamplerA_On == 1)((get(SAMPLER_PLAY_PAD_4)>0) ? sendMidiMessage(0x90, 0x5A, 0x7F) : sendMidiMessage(0x90, 0x5A, 0x00));
	if(SamplerA_On == 1)((get(SAMPLER_PLAY_PAD_5)>0) ? sendMidiMessage(0x90, 0x5B, 0x7F) : sendMidiMessage(0x90, 0x5B, 0x00));
	if(SamplerA_On == 1)((get(SAMPLER_PLAY_PAD_6)>0) ? sendMidiMessage(0x90, 0x5C, 0x7F) : sendMidiMessage(0x90, 0x5C, 0x00));
	if(SamplerA_On == 0)((get(HOT_CUE_1_A)>0) ? sendMidiMessage(0x90, 0x26, 0x7F) : sendMidiMessage(0x90, 0x26, 0x00));
	if(SamplerA_On == 0)((get(HOT_CUE_2_A)>0) ? sendMidiMessage(0x90, 0x27, 0x7F) : sendMidiMessage(0x90, 0x27, 0x00));
	if(SamplerA_On == 0)((get(HOT_CUE_3_A)>0) ? sendMidiMessage(0x90, 0x28, 0x7F) : sendMidiMessage(0x90, 0x28, 0x00));
	(get(SYNC_LOCK_A)>0) ? sendMidiMessage(0x90, 0x31, 0x7F) : sendMidiMessage(0x90, 0x31, 0x00);
	(get(CUE_A)>0) ? sendMidiMessage(0x90, 0x2F, 0x7F) : sendMidiMessage(0x90, 0x2F, 0x00);
	(get(CUE_PLAY_A)>0) ? sendMidiMessage(0x90, 0x30, 0x7F) : sendMidiMessage(0x90, 0x30, 0x00);
	(get(EFFECT_ON_1_GROUP_A)>0) ? sendMidiMessage(0x90, 0x22, 0x7F) : sendMidiMessage(0x90, 0x22, 0x00);
	(get(KEY_LOCK_A)>0) ? sendMidiMessage(0x90, 0x2D, 0x7F) : sendMidiMessage(0x90, 0x2D, 0x00);
	(get(LOOP_A)>0) ? sendMidiMessage(0x90, 0x23, 0x7F) : sendMidiMessage(0x90, 0x23, 0x00); //no get for half/double,cos not states. Light all 3
	(get(LOOP_A)>0) ? sendMidiMessage(0x90, 0x24, 0x7F) : sendMidiMessage(0x90, 0x24, 0x00); 
	(get(LOOP_A)>0) ? sendMidiMessage(0x90, 0x25, 0x7F) : sendMidiMessage(0x90, 0x25, 0x00);  
	(get(PFL_A)>0) ? sendMidiMessage(0x90, 0x32, 0x7F) : sendMidiMessage(0x90, 0x32, 0x00);
	
	//Deck B LEDs
	(get(PLAY_PAUSE_B) > 0) ? sendMidiMessage(0x91, 0x2E, 0x7F) : sendMidiMessage(0x91, 0x2E, 0x00);
	if(SamplerB_On == 1)((get(SAMPLER_PLAY_PAD_7)>0) ? sendMidiMessage(0x91, 0x56, 0x7F) : sendMidiMessage(0x91, 0x56, 0x00));
	if(SamplerB_On == 1)((get(SAMPLER_PLAY_PAD_8)>0) ? sendMidiMessage(0x91, 0x57, 0x7F) : sendMidiMessage(0x91, 0x57, 0x00));
	if(SamplerB_On == 1)((get(SAMPLER_PLAY_PAD_9)>0) ? sendMidiMessage(0x91, 0x58, 0x7F) : sendMidiMessage(0x91, 0x58, 0x00));
	if(SamplerB_On == 1)((get(SAMPLER_PLAY_PAD_10)>0) ? sendMidiMessage(0x91, 0x5A, 0x7F) : sendMidiMessage(0x91, 0x5A, 0x00));
	if(SamplerB_On == 1)((get(SAMPLER_PLAY_PAD_11)>0) ? sendMidiMessage(0x91, 0x5B, 0x7F) : sendMidiMessage(0x91, 0x5B, 0x00));
	if(SamplerB_On == 1)((get(SAMPLER_PLAY_PAD_12)>0) ? sendMidiMessage(0x91, 0x5C, 0x7F) : sendMidiMessage(0x91, 0x5C, 0x00));
	if(SamplerB_On == 0)((get(HOT_CUE_1_B)>0) ? sendMidiMessage(0x91, 0x26, 0x7F) : sendMidiMessage(0x91, 0x26, 0x00));
	if(SamplerB_On == 0)((get(HOT_CUE_2_B)>0) ? sendMidiMessage(0x91, 0x27, 0x7F) : sendMidiMessage(0x91, 0x27, 0x00));
	if(SamplerB_On == 0)((get(HOT_CUE_3_B)>0) ? sendMidiMessage(0x91, 0x28, 0x7F) : sendMidiMessage(0x91, 0x28, 0x00));
	(get(SYNC_LOCK_B)>0) ? sendMidiMessage(0x91, 0x31, 0x7F) : sendMidiMessage(0x91, 0x31, 0x00);
	(get(CUE_B)>0) ? sendMidiMessage(0x91, 0x2F, 0x7F) : sendMidiMessage(0x91, 0x2F, 0x00);
	(get(CUE_PLAY_B)>0) ? sendMidiMessage(0x91, 0x30, 0x7F) : sendMidiMessage(0x91, 0x30, 0x00);
	(get(EFFECT_ON_1_GROUP_B)>0) ? sendMidiMessage(0x91, 0x22, 0x7F) : sendMidiMessage(0x91, 0x22, 0x00);
	(get(KEY_LOCK_B)>0) ? sendMidiMessage(0x91, 0x2D, 0x7F) : sendMidiMessage(0x91, 0x2D, 0x00);
	(get(LOOP_B)>0) ? sendMidiMessage(0x91, 0x23, 0x7F) : sendMidiMessage(0x91, 0x23, 0x00);
	(get(LOOP_B)>0) ? sendMidiMessage(0x91, 0x24, 0x7F) : sendMidiMessage(0x91, 0x24, 0x00); 
	(get(LOOP_B)>0) ? sendMidiMessage(0x91, 0x25, 0x7F) : sendMidiMessage(0x91, 0x25, 0x00);  
	(get(PFL_B)>0) ? sendMidiMessage(0x91, 0x32, 0x7F) : sendMidiMessage(0x91, 0x32, 0x00);
	
	// add a bit of a skew to make the level more obvious
	float level_l = pow(2.7182818, log(get(MASTER_LEVEL_METER_L)) / 3.0);
	float level_r = pow(2.7182818, log(get(MASTER_LEVEL_METER_R)) / 3.0);
	
	//Master VU LEDs (Left)
	(level_l>=1.0) ? sendMidiMessage(0x92, 0x26, 0x7F) : sendMidiMessage(0x92, 0x26, 0x00); //Top LED
	(level_l>=0.875) ? sendMidiMessage(0x92, 0x27, 0x7F) : sendMidiMessage(0x92, 0x27, 0x00);
	(level_l>=0.75) ? sendMidiMessage(0x92, 0x28, 0x7F) : sendMidiMessage(0x92, 0x28, 0x00);
	(level_l>=0.625) ? sendMidiMessage(0x92, 0x29, 0x7F) : sendMidiMessage(0x92, 0x29, 0x00);
	(level_l>=0.5) ? sendMidiMessage(0x92, 0x2A, 0x7F) : sendMidiMessage(0x92, 0x2A, 0x00);
	(level_l>=0.375) ? sendMidiMessage(0x92, 0x2B, 0x7F) : sendMidiMessage(0x92, 0x2B, 0x00);
	(level_l>=0.250) ? sendMidiMessage(0x92, 0x2C, 0x7F) : sendMidiMessage(0x92, 0x2C, 0x00);
	(level_l>=0.125) ? sendMidiMessage(0x92, 0x2D, 0x7F) : sendMidiMessage(0x92, 0x2D, 0x00); //Bottom LED
	
	//Master VU LEDs (Right)
	(level_r>=1.0) ? sendMidiMessage(0x92, 0x2E, 0x7F) : sendMidiMessage(0x92, 0x2E, 0x00); //Top LED
	(level_r>=0.875) ? sendMidiMessage(0x92, 0x2F, 0x7F) : sendMidiMessage(0x92, 0x2F, 0x00);
	(level_r>=0.75) ? sendMidiMessage(0x92, 0x30, 0x7F) : sendMidiMessage(0x92, 0x30, 0x00);
	(level_r>=0.625) ? sendMidiMessage(0x92, 0x31, 0x7F) : sendMidiMessage(0x92, 0x31, 0x00);
	(level_r>=0.5) ? sendMidiMessage(0x92, 0x32, 0x7F) : sendMidiMessage(0x92, 0x32, 0x00);
	(level_r>=0.375) ? sendMidiMessage(0x92, 0x33, 0x7F) : sendMidiMessage(0x92, 0x33, 0x00);
	(level_r>=0.250) ? sendMidiMessage(0x92, 0x34, 0x7F) : sendMidiMessage(0x92, 0x34, 0x00);
	(level_r>=0.125) ? sendMidiMessage(0x92, 0x35, 0x7F) : sendMidiMessage(0x92, 0x35, 0x00); //Bottom LED
	
}

void OnStart()
{
	// not used
}

void OnStop()
{
	// not used
}
